/*
 * Created on Jun 18, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package edu.iastate.jrelm.rl.rotherev.advanced;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;

import com.sun.org.apache.xalan.internal.lib.ExsltBase;


import uchicago.src.reflector.IntrospectPanel;
import uchicago.src.reflector.ListPropertyDescriptor;
import uchicago.src.reflector.PropertyButton;
import uchicago.src.sim.gui.Named;

import edu.iastate.jrelm.core.Action;
import edu.iastate.jrelm.rl.rotherev.REParameters;
import edu.iastate.jrelm.rl.rotherev.variant.VREParameters;
import edu.iastate.jrelm.similarity.DissimilarityMeasure;
import edu.iastate.jrelm.similarity.SimpleDissimilarity;
import edu.iastate.jrelm.spillover.SpilloverByDissimilarity;
import edu.iastate.jrelm.spillover.SpilloverWeightGenerator;


/**
 * Parameters required for the Advanced version of the Roth-Erev reinforcement learning 
 * algorithm. These parameters include setting for features extended from the standard 
 * Roth-Erev algorithm. This includes: 
 *<ul>
 *	<li>Alternative methods to generate Action probabilities.</li>
 *	<li>Alternative methods of reward spillover among similar actions</li>
 *	<li>Ability to specify similar measures for Action comparison
 *</ul>
 *
 * @author Charles Gieseler
 *
 */
public class AREParameters extends REParameters{
	
	protected int numberOfActions = 0;
	
	/* ********************************************************************************
	 * 						PROBABILITY GENERATION FIELDS
	 * ********************************************************************************/

	//Which method to use in generating probabilities for Actions
//	private int selectedProbMethod = ARELearner.USE_BOLTZMAN_PROBABILITY;
			
//	private final double DEFAULT_BOLTZMANN = 9999999.0;
//	private double boltzmannCooling = DEFAULT_BOLTZMANN;
	
	/* ********************************************************************************
	 * 								SPILLOVER FIELDS
	 * ********************************************************************************/
	//Which method of reward spillover to use
	private SpilloverWeightGenerator selectedSpillover;

	//List of available spillover methods
	private List<SpilloverWeightGenerator> spilloverList;
	
	/* ********************************************************************************
	 * 						DESCRIPTOR CONTAINER RELATED FIELDS
	 * ********************************************************************************/
	
	//Array of names of the parameters as Strings
	private String[] paramNames = new String[] {
			"BoltzmannTemp", "Experimentation", "InitialPropensity", "Recency", 
			"RandomSeed", "ActiveSpilloverMethod"};
//		"TestButtonParameter"};

	//Collection of parameter descriptors. Allow parameters to be displayed and
	//edited in the RePast interface.
	Hashtable paramDescriptors = new Hashtable();
	
	PropertyButton testButtonParam;
	/* ********************************************************************************
	 * 									CONSTRUCTORS
	 * ********************************************************************************/
	
	/**
	 *	Construct default AREParameters. This will create settings to
	 *	the same as the default REParameters. Essentially this will run the
	 *	ARELearner as a standard VRELearner with default settings.
	 */
	public AREParameters(){
		super();

		init();
	}
	
	/**
	 * Build a AREParameters object with the given learning parameter values.
	 * Essentially this will run the ARELearner as a standard VRELearner 
	 * with the given settings.
	 */
	public AREParameters(
			double boltzmannTemp,
			double experimentation,
			double initialPropensity,
			int numActions,
			double recency){
		super(boltzmannTemp, experimentation, initialPropensity, recency);
		
		this.numberOfActions = numActions;
		init();		
	}

	/**
	 * 
	 * <P>
	 * @param boltzmannCooling - cooling parameter for gibbs-boltzmann cooling method. 
	 * @param numActions
	 * @param recency
	 * @param experimentation
	 * @param initialPropensity
	 * 
	 * @param spillover 
	 * 
	 * 
	 */
	public AREParameters(
			double boltzmannTemp,
			double experimentation,
			double initialPropensity,
			int numActions, 
			double recency,
			Vector<SpilloverWeightGenerator> spilloverList){
		
		super(boltzmannTemp, experimentation, initialPropensity, recency);
		this.numberOfActions = numActions;
		this.spilloverList = spilloverList;
		
		validateParameters();
		
		init();
	}	
	
	/* ********************************************************************************
	 * 								INITIALIZATION
	 * ********************************************************************************/
	protected void init(){
	
//		buildProbabiltySelector();
		
		if(spilloverList == null)
			spilloverList = new ArrayList<SpilloverWeightGenerator>();
			
		buildSpilloverSelector();
		
		//Experimentation
//		SimpleDissimilarity<Action> measure = new SimpleDissimilarity<Action>();
//		SpilloverByDissimilarity<Action, SimpleDissimilarity<Action>> spill = 
//			new ExponentialSpillover<Action, SimpleDissimilarity<Action>>(measure, -1);
//		testButtonParam = new PropertyButton(spill, true);
//		testButtonParam.setText("Spillover Settings");
	}
	
//	protected void ProbabiltySelector(){
//		Hashtable probTable = new Hashtable();
//		
//		probTable.put(new Integer(ARELearner.USE_RELATIVE_PROPENSITY_PROBABILITY), 
//				"Relative Propensity");
//		
//		probTable.put(new Integer(ARELearner.USE_BOLTZMAN_PROBABILITY), 
//		"Gibbs-Boltzmann");
//		
//		//Build the property descriptor for the GUI
//		ListPropertyDescriptor lpd = new ListPropertyDescriptor("ProbabilityMethod", probTable);
//
//		paramDescriptors.put("ProbabilityMethod", lpd);
//		
//	}
	
	protected void buildSpilloverSelector(){
		Hashtable<SpilloverWeightGenerator, String> optionTable = 
			new Hashtable<SpilloverWeightGenerator, String>();
		
		for(SpilloverWeightGenerator method: spilloverList){
			optionTable.put(method, method.getName());
		}

		//Add no-spillover option if not already present
		NoSpillover noSpill = new NoSpillover();
		if(!optionTable.containsValue(noSpill.getName())){
	
			optionTable.put(noSpill, noSpill.getName());
			spilloverList.add(noSpill);
		}
		
		//Add standard spillover if not already present
		StandardSpillover standard = 
				new StandardSpillover(getExperimentation(), getNumberOfActions());
		
		if(!optionTable.containsValue(standard.getName())){
			optionTable.put(standard, standard.getName());
			spilloverList.add(standard);
		}
		
		//Build the property descriptor for the GUI
		ListPropertyDescriptor lpd   = new ListPropertyDescriptor("SpilloverMethod", optionTable);
		
		paramDescriptors.put("ActiveSpilloverMethod", lpd);
		
		//TODO: select proper method
		selectedSpillover = standard;
//System.out.println(((JComboBox)lpd.getWidget()).getSelectedItem());		
	}
	
	/* *******************************************************
	 * 					ACCESSOR METHODS
	 * *******************************************************/

	public String getName(){
		return "Advanced Roth-Erev";
	}
	
	public int getNumberOfActions(){
		return numberOfActions;
	}
	
	/**
	 * Get the names of the parameters
	 * 
	 * @return Parameter names as an array of Strings
	 */
	public String[] getParameterNames(){
		return paramNames;
	}
	
	public Hashtable getParameterDescriptors(){
		return paramDescriptors;
	}
		
	/**
	 * Get the active SpilloverWeightGenerator indicating the method of spillover 
	 * currently being used.
	 * 
	 * @return currently active SpilloverWeightGenerator 
	 */
	public SpilloverWeightGenerator getActiveSpilloverMethod(){
		return selectedSpillover;
	}
	
	/**
	 * Sets which spillover method to use in ARELearner's wieght function
	 * @param spilloverSelection
	 */
	public void setActiveSpilloverMethod(SpilloverWeightGenerator spilloverSelection){
		//TODO: check for compatibility, perhaps add to the list if it not already there
		this.selectedSpillover = spilloverSelection;
	}
	
	public List<SpilloverWeightGenerator> getSpilloverMethodList(){return spilloverList;}
	
	public void setSpilloverMethodList(List<SpilloverWeightGenerator> spilloverList){
		//TODO: check for compatibility
		this.spilloverList = spilloverList;
		selectedSpillover = spilloverList.get(0);
		
		buildSpilloverSelector();
	}
	
	public void addSpilloverMethod(SpilloverWeightGenerator spilloverOption){
		//TODO: check for compatibility 
		spilloverList.add(spilloverOption);
		
		buildSpilloverSelector();
	}
	
	/**
	 * Overides toSring() to return the name of the aglorithm these parameters are associated with.
	 */
//	public String toString(){return getName();}
	
	
	//**** EXPERIMENTATION ****
//	public PropertyButton getTestButtonParameter(){
//		return testButtonParam;
//	}
//	
//	public void setTestButtonParameter(PropertyButton button){
//		
//		
//	}
}
